@@ -90,42 +90,48 @@ extern int yydebug;
     VAR_REQUEST_XFR = 300,
     VAR_NOTIFY = 301,
     VAR_PROVIDE_XFR = 302,
-    VAR_NOTIFY_RETRY = 303,
-    VAR_OUTGOING_INTERFACE = 304,
-    VAR_ALLOW_AXFR_FALLBACK = 305,
-    VAR_KEY = 306,
-    VAR_ALGORITHM = 307,
-    VAR_SECRET = 308,
-    VAR_AXFR = 309,
-    VAR_UDP = 310,
-    VAR_VERBOSITY = 311,
-    VAR_HIDE_VERSION = 312,
-    VAR_PATTERN = 313,
-    VAR_INCLUDEPATTERN = 314,
-    VAR_ZONELISTFILE = 315,
-    VAR_REMOTE_CONTROL = 316,
-    VAR_CONTROL_ENABLE = 317,
-    VAR_CONTROL_INTERFACE = 318,
-    VAR_CONTROL_PORT = 319,
-    VAR_SERVER_KEY_FILE = 320,
-    VAR_SERVER_CERT_FILE = 321,
-    VAR_CONTROL_KEY_FILE = 322,
-    VAR_CONTROL_CERT_FILE = 323,
-    VAR_XFRDIR = 324,
-    VAR_RRL_SIZE = 325,
-    VAR_RRL_RATELIMIT = 326,
-    VAR_RRL_SLIP = 327,
-    VAR_RRL_IPV4_PREFIX_LENGTH = 328,
-    VAR_RRL_IPV6_PREFIX_LENGTH = 329,
-    VAR_RRL_WHITELIST_RATELIMIT = 330,
-    VAR_RRL_WHITELIST = 331,
-    VAR_ZONEFILES_CHECK = 332,
-    VAR_ZONEFILES_WRITE = 333,
-    VAR_LOG_TIME_ASCII = 334,
-    VAR_ROUND_ROBIN = 335,
-    VAR_ZONESTATS = 336,
-    VAR_REUSEPORT = 337,
-    VAR_VERSION = 338
+    VAR_SIZE_LIMIT_XFR = 303,
+    VAR_NOTIFY_RETRY = 304,
+    VAR_OUTGOING_INTERFACE = 305,
+    VAR_ALLOW_AXFR_FALLBACK = 306,
+    VAR_KEY = 307,
+    VAR_ALGORITHM = 308,
+    VAR_SECRET = 309,
+    VAR_AXFR = 310,
+    VAR_UDP = 311,
+    VAR_VERBOSITY = 312,
+    VAR_HIDE_VERSION = 313,
+    VAR_PATTERN = 314,
+    VAR_INCLUDEPATTERN = 315,
+    VAR_ZONELISTFILE = 316,
+    VAR_REMOTE_CONTROL = 317,
+    VAR_CONTROL_ENABLE = 318,
+    VAR_CONTROL_INTERFACE = 319,
+    VAR_CONTROL_PORT = 320,
+    VAR_SERVER_KEY_FILE = 321,
+    VAR_SERVER_CERT_FILE = 322,
+    VAR_CONTROL_KEY_FILE = 323,
+    VAR_CONTROL_CERT_FILE = 324,
+    VAR_XFRDIR = 325,
+    VAR_RRL_SIZE = 326,
+    VAR_RRL_RATELIMIT = 327,
+    VAR_RRL_SLIP = 328,
+    VAR_RRL_IPV4_PREFIX_LENGTH = 329,
+    VAR_RRL_IPV6_PREFIX_LENGTH = 330,
+    VAR_RRL_WHITELIST_RATELIMIT = 331,
+    VAR_RRL_WHITELIST = 332,
+    VAR_ZONEFILES_CHECK = 333,
+    VAR_ZONEFILES_WRITE = 334,
+    VAR_LOG_TIME_ASCII = 335,
+    VAR_ROUND_ROBIN = 336,
+    VAR_ZONESTATS = 337,
+    VAR_REUSEPORT = 338,
+    VAR_VERSION = 339,
+    VAR_MAX_REFRESH_TIME = 340,
+    VAR_MIN_REFRESH_TIME = 341,
+    VAR_MAX_RETRY_TIME = 342,
+    VAR_MIN_RETRY_TIME = 343,
+    VAR_MULTI_MASTER_CHECK = 344
   };
 #endif
 /* Tokens.  */
@@ -174,42 +180,48 @@ extern int yydebug;
 #define VAR_REQUEST_XFR 300
 #define VAR_NOTIFY 301
 #define VAR_PROVIDE_XFR 302
-#define VAR_NOTIFY_RETRY 303
-#define VAR_OUTGOING_INTERFACE 304
-#define VAR_ALLOW_AXFR_FALLBACK 305
-#define VAR_KEY 306
-#define VAR_ALGORITHM 307
-#define VAR_SECRET 308
-#define VAR_AXFR 309
-#define VAR_UDP 310
-#define VAR_VERBOSITY 311
-#define VAR_HIDE_VERSION 312
-#define VAR_PATTERN 313
-#define VAR_INCLUDEPATTERN 314
-#define VAR_ZONELISTFILE 315
-#define VAR_REMOTE_CONTROL 316
-#define VAR_CONTROL_ENABLE 317
-#define VAR_CONTROL_INTERFACE 318
-#define VAR_CONTROL_PORT 319
-#define VAR_SERVER_KEY_FILE 320
-#define VAR_SERVER_CERT_FILE 321
-#define VAR_CONTROL_KEY_FILE 322
-#define VAR_CONTROL_CERT_FILE 323
-#define VAR_XFRDIR 324
-#define VAR_RRL_SIZE 325
-#define VAR_RRL_RATELIMIT 326
-#define VAR_RRL_SLIP 327
-#define VAR_RRL_IPV4_PREFIX_LENGTH 328
-#define VAR_RRL_IPV6_PREFIX_LENGTH 329
-#define VAR_RRL_WHITELIST_RATELIMIT 330
-#define VAR_RRL_WHITELIST 331
-#define VAR_ZONEFILES_CHECK 332
-#define VAR_ZONEFILES_WRITE 333
-#define VAR_LOG_TIME_ASCII 334
-#define VAR_ROUND_ROBIN 335
-#define VAR_ZONESTATS 336
-#define VAR_REUSEPORT 337
-#define VAR_VERSION 338
+#define VAR_SIZE_LIMIT_XFR 303
+#define VAR_NOTIFY_RETRY 304
+#define VAR_OUTGOING_INTERFACE 305
+#define VAR_ALLOW_AXFR_FALLBACK 306
+#define VAR_KEY 307
+#define VAR_ALGORITHM 308
+#define VAR_SECRET 309
+#define VAR_AXFR 310
+#define VAR_UDP 311
+#define VAR_VERBOSITY 312
+#define VAR_HIDE_VERSION 313
+#define VAR_PATTERN 314
+#define VAR_INCLUDEPATTERN 315
+#define VAR_ZONELISTFILE 316
+#define VAR_REMOTE_CONTROL 317
+#define VAR_CONTROL_ENABLE 318
+#define VAR_CONTROL_INTERFACE 319
+#define VAR_CONTROL_PORT 320
+#define VAR_SERVER_KEY_FILE 321
+#define VAR_SERVER_CERT_FILE 322
+#define VAR_CONTROL_KEY_FILE 323
+#define VAR_CONTROL_CERT_FILE 324
+#define VAR_XFRDIR 325
+#define VAR_RRL_SIZE 326
+#define VAR_RRL_RATELIMIT 327
+#define VAR_RRL_SLIP 328
+#define VAR_RRL_IPV4_PREFIX_LENGTH 329
+#define VAR_RRL_IPV6_PREFIX_LENGTH 330
+#define VAR_RRL_WHITELIST_RATELIMIT 331
+#define VAR_RRL_WHITELIST 332
+#define VAR_ZONEFILES_CHECK 333
+#define VAR_ZONEFILES_WRITE 334
+#define VAR_LOG_TIME_ASCII 335
+#define VAR_ROUND_ROBIN 336
+#define VAR_ZONESTATS 337
+#define VAR_REUSEPORT 338
+#define VAR_VERSION 339
+#define VAR_MAX_REFRESH_TIME 340
+#define VAR_MIN_REFRESH_TIME 341
+#define VAR_MAX_RETRY_TIME 342
+#define VAR_MIN_RETRY_TIME 343
+#define VAR_MULTI_MASTER_CHECK 344
 
 /* Value type.  */
 #if ! defined YYSTYPE && ! defined YYSTYPE_IS_DECLARED
@@ -220,7 +232,7 @@ union YYSTYPE
 
 	char*	str;
 
-#line 224 "configparser.h" /* yacc.c:1909  */
+#line 236 "configparser.h" /* yacc.c:1909  */
 };
 
 typedef union YYSTYPE YYSTYPE;
