@@ -953,7 +953,10 @@ zparser_conv_loc(region_type *region, char *str)
 	}
 
 	/* Meters of altitude... */
-	(void) strtol(str, &str, 10);
+	if(strtol(str, &str, 10) == LONG_MAX) {
+		zc_error_prev_line("altitude too large, number overflow");
+		return NULL;
+	}
 	switch(*str) {
 	case ' ':
 	case '\0':
@@ -1576,21 +1579,21 @@ zonec_read(const char* name, const char* zonefile, zone_type* zone)
 	dname = dname_parse(parser->rr_region, name);
 	if (!dname) {
 		zc_error("incorrect zone name '%s'", name);
-		return 0;
+		return 1;
 	}
 
 #ifndef ROOT_SERVER
 	/* Is it a root zone? Are we a root server then? Idiot proof. */
 	if (dname->label_count == 1) {
 		zc_error("not configured as a root server");
-		return 0;
+		return 1;
 	}
 #endif
 
 	/* Open the zone file */
 	if (!zone_open(zonefile, 3600, CLASS_IN, dname)) {
 		zc_error("cannot open '%s': %s", zonefile, strerror(errno));
-		return 0;
+		return 1;
 	}
 	parser->current_zone = zone;
 
