@@ -119,18 +119,19 @@ extern int yydebug;
     T_CAA = 329,
     T_CDS = 330,
     T_CDNSKEY = 331,
-    T_CSYNC = 332,
-    DOLLAR_TTL = 333,
-    DOLLAR_ORIGIN = 334,
-    NL = 335,
-    SP = 336,
-    STR = 337,
-    PREV = 338,
-    BITLAB = 339,
-    T_TTL = 340,
-    T_RRCLASS = 341,
-    URR = 342,
-    T_UTYPE = 343
+    T_OPENPGPKEY = 332,
+    T_CSYNC = 333,
+    DOLLAR_TTL = 334,
+    DOLLAR_ORIGIN = 335,
+    NL = 336,
+    SP = 337,
+    STR = 338,
+    PREV = 339,
+    BITLAB = 340,
+    T_TTL = 341,
+    T_RRCLASS = 342,
+    URR = 343,
+    T_UTYPE = 344
   };
 #endif
 /* Tokens.  */
@@ -208,18 +209,19 @@ extern int yydebug;
 #define T_CAA 329
 #define T_CDS 330
 #define T_CDNSKEY 331
-#define T_CSYNC 332
-#define DOLLAR_TTL 333
-#define DOLLAR_ORIGIN 334
-#define NL 335
-#define SP 336
-#define STR 337
-#define PREV 338
-#define BITLAB 339
-#define T_TTL 340
-#define T_RRCLASS 341
-#define URR 342
-#define T_UTYPE 343
+#define T_OPENPGPKEY 332
+#define T_CSYNC 333
+#define DOLLAR_TTL 334
+#define DOLLAR_ORIGIN 335
+#define NL 336
+#define SP 337
+#define STR 338
+#define PREV 339
+#define BITLAB 340
+#define T_TTL 341
+#define T_RRCLASS 342
+#define URR 343
+#define T_UTYPE 344
 
 /* Value type.  */
 #if ! defined YYSTYPE && ! defined YYSTYPE_IS_DECLARED
@@ -236,7 +238,7 @@ union YYSTYPE
 	uint16_t	  type;
 	uint16_t	 *unknown;
 
-#line 240 "zparser.h" /* yacc.c:1909  */
+#line 242 "zparser.h" /* yacc.c:1909  */
 };
 
 typedef union YYSTYPE YYSTYPE;
